import shlex
import subprocess
import time
from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass

import requests
from thehive4py.client import TheHiveApi
from thehive4py.query.filters import Eq


@dataclass
class Container:
    url: str
    name: str


def is_hive_container_responsive(container_url: str) -> bool:
    COOLDOWN = 1.0
    TIMEOUT = 30.0

    now = time.time()
    end = now + TIMEOUT

    while now < end:
        try:
            response = requests.get(f"{container_url}/api/status")
            if response.ok:
                return True
        except Exception:
            pass

        time.sleep(COOLDOWN)
        now = time.time()
    return False


def is_hive_container_exist(container_name: str) -> bool:
    exist_cmd = subprocess.run(
        shlex.split(f"docker ps -aq --filter name={container_name}"),
        capture_output=True,
        text=True,
    )
    return bool(exist_cmd.stdout.strip())


def generate_container_name() -> str:
    CONTAINER_PREFIX = "thehive4py-integration-tests"
    return CONTAINER_PREFIX


def get_container_port(container_name: str) -> str:
    port_cmd = subprocess.run(
        shlex.split(f"docker port {container_name}"), capture_output=True, text=True
    )
    port = port_cmd.stdout.strip().split(":")[-1]
    return port


def build_container_url(container_name: str) -> str:
    port = get_container_port(container_name)
    return f"http://localhost:{port}"


def run_hive_container(
    container_name: str, container_image: str = "thehive4py-thehive:5.0.23"
):
    subprocess.run(
        shlex.split(
            f"docker run -d --rm -p 9000 --name {container_name} {container_image}"
        ),
        capture_output=True,
        text=True,
    )


def cleanup_hive_container(container_name: str):
    subprocess.run(
        shlex.split(f"docker rm -f {container_name}"),
        capture_output=True,
        text=True,
    )


def spawn_hive_container() -> Container:
    name = generate_container_name()

    if not is_hive_container_exist(container_name=name):
        run_hive_container(container_name=name)
    url = build_container_url(container_name=name)

    if not is_hive_container_responsive(container_url=url):
        cleanup_hive_container(container_name=name)
        raise RuntimeError("Unable to startup test container for TheHive")

    return Container(url=url, name=name)


def reinit_hive_container(client: TheHiveApi) -> None:

    original_session_organisation = client.session_organisation
    for organisation in ["test-org", "share-org", "admin"]:
        client.session_organisation = organisation
        alerts = client.alert.find()
        cases = client.case.find()
        users = client.user.find(filters=~Eq("login", "admin@thehive.local"))
        profiles = client.profile.find(
            filters=~Eq("name", "analyst") & ~Eq("name", "read-only")
        )
        observable_types = client.observable_type.find(
            filters=~Eq("_createdBy", "system@thehive.local")
        )
        custom_fields = client.custom_field.list()
        with ThreadPoolExecutor() as executor:
            executor.map(client.alert.delete, [alert["_id"] for alert in alerts])
            executor.map(client.case.delete, [case["_id"] for case in cases])
            executor.map(client.user.delete, [user["_id"] for user in users])
            executor.map(
                client.profile.delete, [profile["_id"] for profile in profiles]
            )
            executor.map(
                client.custom_field.delete,
                [custom_field["_id"] for custom_field in custom_fields],
            )
            executor.map(
                client.observable_type.delete,
                [observable_type["_id"] for observable_type in observable_types],
            )

    client.session_organisation = original_session_organisation



from typing import List, Optional, Sequence, Union

from thehive4py.endpoints._base import EndpointBase
from thehive4py.query import QueryExpr
from thehive4py.query.filters import FilterExpr
from thehive4py.query.page import Paginate
from thehive4py.query.sort import SortExpr
from thehive4py.types.attachment import OutputAttachment
from thehive4py.types.case import (
    CaseStatus,
    CaseStatusValue,
    ImpactStatusValue,
    InputBulkUpdateCase,
    InputCase,
    InputImportCase,
    InputUpdateCase,
    OutputCase,
)
from thehive4py.types.comment import OutputComment
from thehive4py.types.observable import InputObservable, OutputObservable
from thehive4py.types.procedure import InputProcedure, OutputProcedure
from thehive4py.types.share import InputShare, OutputShare
from thehive4py.types.task import InputTask, OutputTask
from thehive4py.types.timeline import OutputTimeline

CaseId = Union[str, int]


class CaseEndpoint(EndpointBase):
    def create(self, case: InputCase) -> OutputCase:
        return self._session.make_request("POST", path="/api/v1/case", json=case)

    def get(self, case_id: CaseId) -> OutputCase:
        return self._session.make_request("GET", path=f"/api/v1/case/{case_id}")

    def delete(self, case_id: CaseId) -> None:
        self._session.make_request("DELETE", path=f"/api/v1/case/{case_id}")

    def update(self, case_id: CaseId, case: InputUpdateCase) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/case/{case_id}", json=case
        )

    def bulk_update(self, fields: InputBulkUpdateCase) -> None:
        return self._session.make_request(
            "PATCH", path="/api/v1/case/_bulk", json=fields
        )

    def merge(self, case_ids: Sequence[CaseId]) -> OutputCase:
        case_id_subpath = ",".join([str(case_id) for case_id in case_ids])
        return self._session.make_request(
            "POST", path=f"/api/v1/case/_merge/{case_id_subpath}"
        )

    def unlink_alert(self, case_id: str, alert_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/case/{case_id}/alert/{alert_id}"
        )

    def merge_similar_observables(self, case_id: CaseId) -> dict:
        # TODO: add better return value type hint
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/observable/_merge"
        )

    def get_linked_cases(self, case_id: CaseId) -> List[OutputCase]:
        return self._session.make_request("GET", path=f"/api/v1/case/{case_id}/links")

    def delete_custom_field(self, custom_field_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/case/customField/{custom_field_id}"
        )

    def import_from_file(self, import_case: InputImportCase, import_path: str) -> dict:
        # TODO: add better return type hints
        return self._session.make_request(
            "POST",
            path="/api/v1/case/import",
            data=import_case,
            files={
                "file": self._fileinfo_from_filepath(import_path),
            },
        )

    def export_to_file(self, case_id: CaseId, password: str, export_path: str) -> None:
        return self._session.make_request(
            "GET",
            path=f"/api/v1/case/{case_id}/export",
            params={"password": password},
            download_path=export_path,
        )

    def get_timeline(self, case_id: CaseId) -> OutputTimeline:
        return self._session.make_request("GET", f"/api/v1/case/{case_id}/timeline")

    def add_attachment(
        self, case_id: CaseId, attachment_paths: List[str]
    ) -> List[OutputAttachment]:
        files = [
            ("attachments", self._fileinfo_from_filepath(attachment_path))
            for attachment_path in attachment_paths
        ]
        return self._session.make_request(
            "POST", f"/api/v1/case/{case_id}/attachments", files=files
        )["attachments"]

    def download_attachment(
        self, case_id: CaseId, attachment_id: str, attachment_path: str
    ) -> None:
        return self._session.make_request(
            "GET",
            path=f"/api/v1/case/{case_id}/attachment/{attachment_id}/download",
            download_path=attachment_path,
        )

    def delete_attachment(self, case_id: CaseId, attachment_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/case/{case_id}/attachment/{attachment_id}"
        )

    def list_shares(self, case_id: CaseId) -> List[OutputShare]:
        return self._session.make_request("GET", path=f"/api/v1/case/{case_id}/shares")

    def share(self, case_id: CaseId, shares: List[InputShare]) -> List[OutputShare]:
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/shares", json={"shares": shares}
        )

    def unshare(self, case_id: CaseId, organisation_ids: List[str]) -> None:
        return self._session.make_request(
            "DELETE",
            path=f"/api/v1/case/{case_id}/shares",
            json={"organisations": organisation_ids},
        )

    def set_share(self, case_id: CaseId, shares: List[InputShare]) -> List[OutputShare]:
        return self._session.make_request(
            "PUT", path=f"/api/v1/case/{case_id}/shares", json={"shares": shares}
        )

    def remove_share(self, share_id: str) -> None:
        return self._session.make_request(
            "DELETE", path=f"/api/v1/case/share/{share_id}"
        )

    def update_share(self, share_id: str, profile: str) -> None:
        return self._session.make_request(
            "PATCH", path=f"/api/v1/case/share/{share_id}", json={"profile": profile}
        )

    def find(
        self,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputCase]:
        query: QueryExpr = [
            {"_name": "listCase"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "cases"},
            json={"query": query},
        )

    def count(self, filters: Optional[FilterExpr] = None) -> int:
        query: QueryExpr = [
            {"_name": "listCase"},
            *self._build_subquery(filters=filters),
            {"_name": "count"},
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "cases.count"},
            json={"query": query},
        )

    def create_task(self, case_id: CaseId, task: InputTask) -> OutputTask:
        return self._session.make_request(
            "POST",
            path=f"/api/v1/case/{case_id}/task",
            json=task,
        )

    def find_tasks(
        self,
        case_id: CaseId,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputTask]:
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "tasks"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-tasks"},
            json={"query": query},
        )

    def create_observable(
        self,
        case_id: CaseId,
        observable: InputObservable,
        observable_path: Optional[str] = None,
    ) -> List[OutputObservable]:
        kwargs = self._build_observable_kwargs(
            observable=observable, observable_path=observable_path
        )
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/observable", **kwargs
        )

    def find_observables(
        self,
        case_id: CaseId,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputObservable]:
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "observables"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-observables"},
            json={"query": query},
        )

    def create_procedure(
        self, case_id: str, procedure: InputProcedure
    ) -> OutputProcedure:
        return self._session.make_request(
            "POST", path=f"/api/v1/case/{case_id}/procedure", json=procedure
        )

    def find_procedures(
        self,
        case_id: str,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputProcedure]:
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "procedures"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]

        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-procedures"},
            json={"query": query},
        )

    def find_attachments(
        self,
        case_id: CaseId,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputAttachment]:
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "attachments"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-attachments"},
            json={"query": query},
        )

    def find_comments(
        self,
        case_id: CaseId,
        filters: Optional[FilterExpr] = None,
        sortby: Optional[SortExpr] = None,
        paginate: Optional[Paginate] = None,
    ) -> List[OutputComment]:
        query: QueryExpr = [
            {"_name": "getCase", "idOrName": case_id},
            {"_name": "comments"},
            *self._build_subquery(filters=filters, sortby=sortby, paginate=paginate),
        ]
        return self._session.make_request(
            "POST",
            path="/api/v1/query",
            params={"name": "case-comments"},
            json={"query": query},
        )

    def close(
        self,
        case_id: CaseId,
        status: CaseStatusValue,
        summary: str,
        impact_status: ImpactStatusValue = "NotApplicable",
    ) -> None:
        case: InputUpdateCase = {
            "status": status,
            "impactStatus": impact_status,
            "summary": summary,
        }
        return self.update(
            case_id,
            case,
        )

    def open(
        self, case_id: CaseId, status: CaseStatusValue = CaseStatus.InProgress
    ) -> None:
        case: InputUpdateCase = {"status": status}
        return self.update(case_id, case)



#!/usr/bin/env python
import argparse
import subprocess


def _run_subprocess(
    args: str,
    init_message: str,
    success_message: str,
    error_message: str,
    verbose=False,
):
    print(init_message)
    proc = subprocess.run(args, shell=True, capture_output=True)

    process_output = proc.stdout.decode() or proc.stderr.decode()
    indented_process_output = "\n".join(
        [f"\t{output_line}" for output_line in process_output.splitlines()]
    )

    if proc.returncode != 0:
        exit_message = "\n".join([error_message, indented_process_output])
        exit(exit_message)

    if verbose:
        print(indented_process_output)

    print(success_message)


def run_all(verbose=False):
    print("Run all deployment tasks...")
    run_build(verbose=verbose)
    run_publish(verbose=verbose)
    print("All tasks succeeded!")


def run_build(verbose: bool):
    _run_subprocess(
        args=("rm -rf build/ dist/ && python -m build --sdist --wheel"),
        init_message="Building the package with the build module...",
        success_message="Package build succeeded!",
        error_message="Package build failed due to:",
        verbose=verbose,
    )


def run_publish(verbose: bool):
    _run_subprocess(
        args=("echo 'Publish command is not implemented yet...' && exit 1 "),
        init_message="Publishing the package with twine...",
        success_message="Publish succeeded!",
        error_message="Publish failed due to:",
        verbose=verbose,
    )


def parse_arguments():
    main_parser = argparse.ArgumentParser(
        prog="thehive4py-cd",
        description="run all cd tasks or use sub commands to run cd tasks individually",
    )
    main_parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        default=False,
        help="generate verbose output",
    )
    main_parser.set_defaults(func=run_all)

    subparsers = main_parser.add_subparsers(help="commands")
    subparser_options = [
        {
            "name": "build",
            "help": "task to build the package",
            "default_func": run_build,
        },
        {
            "name": "publish",
            "help": "task to publish the package",
            "default_func": run_publish,
        },
    ]

    for subparser_option in subparser_options:
        _subparser = subparsers.add_parser(
            name=subparser_option["name"],
            help=subparser_option["help"],
            parents=[main_parser],
            add_help=False,
        )
        _subparser.set_defaults(func=subparser_option["default_func"])

    return main_parser.parse_args()


def main():
    args = parse_arguments()
    args.func(verbose=args.verbose)


if __name__ == "__main__":
    main()



from collections import UserDict


class FilterExpr(UserDict):
    """Base class for filter expressions."""

    def __and__(self, other: "FilterExpr") -> "FilterExpr":
        if not isinstance(other, FilterExpr):
            self._raise_type_error("&", self, other)
        args = self.get("_and", [self]) + other.get("_and", [other])
        return FilterExpr(_and=args)

    def __or__(self, other: "FilterExpr") -> "FilterExpr":  # type:ignore
        if not isinstance(other, FilterExpr):
            self._raise_type_error("|", self, other)
        args = self.get("_or", [self]) + other.get("_or", [other])
        return FilterExpr(_or=args)

    def __invert__(self) -> "FilterExpr":
        return FilterExpr(_not=self)

    def _raise_type_error(self, operand, first, second):
        raise TypeError(
            f"unsupported operand type(s) for {operand}: "
            f"{type(first)} and {type(second)}"
        )


class Eq(FilterExpr):
    def __init__(self, field: str, value):
        super().__init__(_field=field, _value=value)


class Like(FilterExpr):
    def __init__(self, field: str, value):
        super().__init__(_like={"_field": field, "_value": value})


class Match(FilterExpr):
    def __init__(self, field: str, value):
        super().__init__(_match={"_field": field, "_value": value})


class StartsWith(FilterExpr):
    def __init__(self, field: str, value):
        super().__init__(_startsWith={"_field": field, "_value": value})


class EndsWith(FilterExpr):
    def __init__(self, field: str, value):
        super().__init__(_endsWith={"_field": field, "_value": value})


class Contains(FilterExpr):
    def __init__(self, field: str):
        super().__init__(_contains=field)


class Gt(FilterExpr):
    def __init__(self, field: str, value):
        super().__init__(_gt={field: value})


class Gte(FilterExpr):
    def __init__(self, field: str, value):
        super().__init__(_gte={field: value})


class Lt(FilterExpr):
    def __init__(self, field: str, value):
        super().__init__(_lt={field: value})


class Lte(FilterExpr):
    def __init__(self, field: str, value):
        super().__init__(_lte={field: value})


class Between(FilterExpr):
    def __init__(self, field: str, start: int, end: int):
        super().__init__(_between={"_field": field, "_from": start, "_to": end})


class Before(FilterExpr):
    def __init__(self, field: str, end: int):
        super().__init__(_between={"_field": field, "_to": end})


class After(FilterExpr):
    def __init__(self, field: str, start: int):
        super().__init__(_between={"_field": field, "_from": start})



#!/usr/bin/env python
import argparse
import subprocess


def _run_subprocess(
    args: str,
    init_message: str,
    success_message: str,
    error_message: str,
    verbose=False,
):
    print(init_message)
    proc = subprocess.run(args, shell=True, capture_output=True)

    process_output = proc.stdout.decode() or proc.stderr.decode()
    indented_process_output = "\n".join(
        [f"\t{output_line}" for output_line in process_output.splitlines()]
    )

    if proc.returncode != 0:
        exit_message = "\n".join([error_message, indented_process_output])
        exit(exit_message)

    if verbose:
        print(indented_process_output)

    print(success_message)


def check_all(verbose=False):
    print("Run all checks...")
    check_lint(verbose=verbose)
    check_format(verbose=verbose)
    check_type(verbose=verbose)
    check_cve(verbose=verbose)
    check_security(verbose=verbose)
    print("All checks succeeded!")


def check_lint(verbose=False):
    _run_subprocess(
        args="flake8 thehive4py/ tests/",
        init_message="Run lint checks with flake8...",
        success_message="Lint checks succeeded!",
        error_message="Lint checks failed due to:",
        verbose=verbose,
    )


def check_format(verbose=False):
    _run_subprocess(
        args="black --diff thehive4py/ tests/",
        init_message="Run format checks with black...",
        success_message="Format checks succeeded!",
        error_message="Lint checks failed due to:",
        verbose=verbose,
    )


def check_type(verbose=False):
    _run_subprocess(
        args="mypy --install-types --non-interactive thehive4py/",
        init_message="Run type checks with mypy...",
        success_message="Type checks succeeded!",
        error_message="Type checks failed due to:",
        verbose=verbose,
    )


def check_cve(verbose=False):
    _run_subprocess(
        args="pip-audit .",
        init_message="Run CVE checks with pip-audit...",
        success_message="CVE checks succeeded!",
        error_message="CVE checks failed due to:",
        verbose=verbose,
    )


def check_security(verbose=False):
    _run_subprocess(
        args="bandit -r thehive4py/",
        init_message="Run security checks with bandit...",
        success_message="Security checks succeeded!",
        error_message="Security checks failed due to:",
        verbose=verbose,
    )


def parse_arguments():
    main_parser = argparse.ArgumentParser(
        prog="thehive4py-ci",
        description=(
            "run all ci checks or use sub commands to run ci checks individually"
        ),
    )
    main_parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        default=False,
        help="generate verbose output",
    )
    main_parser.set_defaults(func=check_all)

    subparsers = main_parser.add_subparsers(help="commands")
    subparser_options = [
        {"name": "lint", "help": "run lint checks only", "default_func": check_lint},
        {
            "name": "format",
            "help": "run format checks only",
            "default_func": check_format,
        },
        {"name": "type", "help": "run type checks only", "default_func": check_type},
        {"name": "cve", "help": "run cve checks only", "default_func": check_cve},
        {
            "name": "security",
            "help": "run security checks",
            "default_func": check_security,
        },
    ]

    for subparser_option in subparser_options:
        _subparser = subparsers.add_parser(
            name=subparser_option["name"],
            help=subparser_option["help"],
            parents=[main_parser],
            add_help=False,
        )
        _subparser.set_defaults(func=subparser_option["default_func"])

    return main_parser.parse_args()


def main():
    args = parse_arguments()
    args.func(verbose=args.verbose)


if __name__ == "__main__":
    main()



from typing import List, Union

from .filters import Between, Contains, Eq  # noqa
from .filters import FilterExpr as _FilterExpr  # noqa
from .filters import Gt, Gte, Like, Lt, Lte  # noqa
from .filters import Match, Before, After, StartsWith, EndsWith  # noqa
from .page import Paginate
from .sort import Asc, Desc  # noqa
from .sort import SortExpr as _SortExpr

QueryExpr = List[Union[_FilterExpr, _SortExpr, Paginate, dict]]
